@@ -507,7 +507,11 @@ char* fcitx_utils_get_process_name()
     /* } while(0); */
     /* return strdup(""); */
 #elif defined(LIBKVM_FOUND)
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+    kvm_t *vm = kvm_open(NULL, NULL, NULL, KVM_NO_FILES, NULL);
+#else
     kvm_t *vm = kvm_open(0, "/dev/null", 0, O_RDONLY, NULL);
+#endif
     if (vm == 0)
         return strdup("");
 
@@ -515,16 +519,28 @@ char* fcitx_utils_get_process_name()
     do {
         int cnt;
         int mypid = getpid();
+#ifdef __NetBSD__
+        struct kinfo_proc2 * kp = kvm_getproc2(vm, KERN_PROC_PID, mypid, sizeof(struct kinfo_proc2), &cnt);
+#else
         struct kinfo_proc * kp = kvm_getprocs(vm, KERN_PROC_PID, mypid, &cnt);
+#endif
         if ((cnt != 1) || (kp == 0)) {
             break;
         }
         int i;
         for (i = 0; i < cnt; i++)
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+            if (kp->p_pid == mypid)
+#else
             if (kp->ki_pid == mypid)
+#endif
                 break;
         if (i != cnt)
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+            result = strdup(kp->p_comm);
+#else
             result = strdup(kp->ki_comm);
+#endif
     } while (0);
     kvm_close(vm);
     if (result == NULL)
@@ -574,6 +590,11 @@ char* fcitx_utils_get_fcitx_path(const char* type)
         }
         else
             result = strdup(LOCALEDIR);
+    } else if (strcmp(type, "tabledir") == 0) {
+        if (fcitxdir) {
+            fcitx_utils_alloc_cat_str(result, fcitxdir, "/share/"PACKAGE"/table");
+        } else
+            result = strdup(PKGDATADIR"/table");
     }
     return result;
 }
@@ -980,6 +1001,58 @@ fcitx_utils_set_escape_str_with_set(char *res, const char *str, const char *set)
     return res;
 }
 
+FCITX_EXPORT_API int fcitx_utils_judge_implugin_service_exist() {
+    FILE *fp;
+    int count;
+    char buf[150];
+    char command[150];
+
+    sprintf(command,
+            "ps -ef | grep fcitx-implugin-service | grep -v grep | wc -l");
+
+    if ((fp = popen(command, "r")) == NULL)
+        return 0;
+
+    if ((fgets(buf, 150, fp)) != NULL) {
+        count = atoi(buf);
+        if (count != 0) {
+            pclose(fp);
+            return 1;
+        }
+    }
+    pclose(fp);
+    return 0;
+}
+FCITX_EXPORT_API int fcitx_utils_kill_fcitx_config_gtk3() {
+    FILE *fp;
+    int pid;
+    char unused[150];
+    char buf[150];
+    char command[150];
+
+    sprintf(command,
+            "ps -ef | grep fcitx-config-gtk3 | grep -v grep");
+
+    if ((fp = popen(command, "r")) == NULL)
+        return 0;
+
+    if ((fgets(buf, 150, fp)) != NULL) {
+        sscanf(buf, "%s\t%d\t%s", unused, &pid, unused);
+    }
+    pclose(fp);
+
+    if(pid == 32767)
+        return 0;
+
+    sprintf(command, "kill -9 %d", pid);
+
+    if ((fp = popen(command, "r")) == NULL)
+        return 0;
+
+    pclose(fp);
+    return 1;
+}
+
 #ifdef __FCITX_ATOMIC_USE_SYNC_FETCH
 /**
  * Also define lib function when there is builtin function for
