@@ -471,6 +471,13 @@ extern "C" {
      **/
     void fcitx_utils_launch_restart(void);
 
+    /**
+     * helper function to execute in place
+     *
+     * @return void
+     **/
+    void fcitx_utils_restart_in_place(void);
+
     /**
      * @brief launch a process
      *
@@ -493,7 +500,7 @@ extern "C" {
     void fcitx_utils_backtrace();
 
     /**
-     * @brief get bool environment var for convinience.
+     * @brief get bool environment var for convenience.
      *
      * @param name var name
      * @param defval default value
@@ -527,7 +534,7 @@ extern "C" {
     int fcitx_utils_strcmp0(const char* a, const char* b);
 
     /**
-     * similiar with fcitx_utils_strcmp0, but empty string will be considered
+     * similar with fcitx_utils_strcmp0, but empty string will be considered
      * equals to NULL in this case.
      * NULL == empty, and empty < not empty
      *
@@ -805,6 +812,9 @@ extern "C" {
         return len;
     }
 
+int fcitx_utils_judge_implugin_service_exist();
+int fcitx_utils_kill_fcitx_config_gtk3();
+
 #ifdef __GCC_HAVE_SYNC_COMPARE_AND_SWAP_4
 #define __FCITX_ATOMIC_USE_SYNC_FETCH
 #elif defined __has_builtin
