@@ -42,7 +42,7 @@
  * If a key event goes through all phase and still the state is IRV_TO_PROCESS, it will be forwarded.
  *
  * When it comes to update candidates, if the flag contains IRV_FLAG_UPDATE_CANDIDATE_WORDS, it will trigger the GetCandWords
- * of input method engine and clean up all stiring in the input window, after that, it will trigger update candidates hook.
+ * of input method engine and clean up all strings in the input window, after that, it will trigger update candidates hook.
  *
  * Key blocker is useful if you want to do something in the post input phase, but you don't want forward key if they do nothing.
  * There is an default implemention inside fcitx, it will blocks key when raw input buffer is not empty. Those keys
@@ -160,6 +160,8 @@ extern "C" {
          * want.
          *
          * And no matter in which case, Reset will be called after that.
+         *
+         * CET_ChangeByUser will not be emitted once CET_ChangeByInactivate is emitted.
          */
         CET_ChangeByInactivate,
         /**
@@ -180,9 +182,10 @@ extern "C" {
         /**
          * when user switch to a different input method by hand
          * such as ctrl+shift by default, or by ui,
-         * not implemented yet, default behavior is reset IM.
+         * default behavior is reset IM.
          */
-        CET_ChangeByUser,
+        CET_SwitchIM,
+        CET_ChangeByUser = CET_SwitchIM, // the old name is not accurate, but keep for compatible.
     } FcitxIMCloseEventType;
 
     typedef boolean(*FcitxIMInit)(void *arg); /**< FcitxIMInit */
@@ -195,6 +198,7 @@ extern "C" {
     typedef INPUT_RETURN_VALUE (*FcitxIMKeyBlocker)(void* arg, FcitxKeySym, unsigned int); /**< FcitxIMKeyBlocker */
     typedef void (*FcitxIMUpdateSurroundingText)(void* arg); /**< FcitxIMKeyBlocker */
     typedef void (*FcitxIMOnClose)(void* arg, FcitxIMCloseEventType);
+    typedef const char *(*FcitxIMGetSubModeName)(void* arg);
 
     /**
      * a more fexible interface for input method
@@ -213,7 +217,8 @@ extern "C" {
         FcitxIMUpdateSurroundingText UpdateSurroundingText; /**< surrounding text update trigger */
         FcitxIMDoInput DoReleaseInput; /**< process key release event */
         FcitxIMOnClose OnClose; /**< process when im being switched away */
-        void* padding[62]; /**< padding */
+        FcitxIMGetSubModeName GetSubModeName; /**< return a string owned by im */
+        void* padding[61]; /**< padding */
     } FcitxIMIFace;
 
     /**
@@ -296,7 +301,8 @@ extern "C" {
 
         FcitxIMOnClose OnClose;
 
-        void* padding[8]; /**< padding */
+        FcitxIMGetSubModeName GetSubModeName; /**< return a string owned by im */
+        void* padding[7]; /**< padding */
     } FcitxIM;
 
     /** a key event is press or release */
@@ -343,6 +349,15 @@ extern "C" {
      **/
     struct _FcitxIM* FcitxInstanceGetCurrentIM(struct _FcitxInstance *instance);
 
+    /**
+     * get input method for ic, return result can be NULL.
+     *
+     * @param instance fcitx instance
+     * @param ic input context
+     * @return _FcitxIM*
+     */
+    struct _FcitxIM* FcitxInstanceGetIM(struct _FcitxInstance *instance, struct _FcitxInputContext* ic);
+
     /**
      * get input method by name
      *
@@ -412,6 +427,17 @@ extern "C" {
      **/
     void FcitxInstanceResetInput(struct _FcitxInstance* instance);
 
+    /**
+     * send a close event, which can be used by standalone module which will do something link reset first.
+     *
+     * @param instance fcitx instance
+     * @param closeEvent close event type
+     * @return void
+     *
+     * @since 4.2.8.4
+     **/
+    void FcitxInstanceSendCloseEvent(struct _FcitxInstance* instance, FcitxIMCloseEventType closeEvent);
+
     /**
      * clean whole input window
      *
@@ -555,6 +581,17 @@ extern "C" {
         struct _FcitxInstance* instance,
         int index);
 
+    /**
+     * @brief choose candidate total by index
+     *
+     * @param instance instance
+     * @param index idx
+     * @return INPUT_RETURN_VALUE
+     **/
+    void FcitxInstanceChooseCandidateByTotalIndex(
+        struct _FcitxInstance* instance,
+        int index);
+
     /**
      * send a new key event to client
      *
@@ -935,6 +972,17 @@ extern "C" {
      */
     void FcitxInstanceUnregisterIM(struct _FcitxInstance* instance, const char* name);
 
+    /**
+     * show current input method information, notably it will use GetSubModeName to show the
+     * sub mode name
+     *
+     * There need to be no other aux preedit string, otherwise it will be rejected.
+     *
+     * @param instance Fcitx Instance
+     * @since 4.2.9.2
+     */
+    void FcitxInstanceShowCurrentIMInfo(struct _FcitxInstance* instance);
+
 #ifdef __cplusplus
 }
 #endif
