@@ -243,13 +243,25 @@ void FcitxDBusMenuDoEvent(void* arg)
                         fcitx_utils_start_process(args);
                     }
                     break;
-                case 5:
-                    fcitx_utils_launch_configure_tool();
+                case 4:
+                    {
+                        FcitxIM* im = FcitxInstanceGetCurrentIM(instance);
+                        if (im && im->owner) {
+                            fcitx_utils_launch_configure_tool_for_addon(im->uniqueName);
+                        }
+                        else {
+                            fcitx_utils_launch_configure_tool();
+                        }
+                    }
                     break;
+//                case 5:
+//                    fcitx_utils_launch_configure_tool();
+//                    break;
                 case 6:
                     FcitxInstanceRestart(instance);
                     break;
                 case 7:
+                    fcitx_utils_kill_fcitx_config_gtk3();
                     FcitxInstanceEnd(instance);
                     break;
             }
@@ -366,18 +378,19 @@ void FcitxDBusMenuFillProperty(FcitxNotificationItem* notificationitem, int32_t
                 case 2:
                 case 3:
                 case 8:
+                case 5:
                     value = "separator";
                     FcitxDBusMenuAppendProperty(&sub, properties, "type", DBUS_TYPE_STRING, &value);
                     break;
-                case 5:
-                    value = _("Configure");
-                    FcitxDBusMenuAppendProperty(&sub, properties, "label", DBUS_TYPE_STRING, &value);
-                    /* this icon sucks on KDE, why configure doesn't have "configure" */
-#if 0
-                    value = "preferences-system";
-                    FcitxDBusMenuAppendProperty(&sub, properties, "icon-name", DBUS_TYPE_STRING, &value);
-#endif
-                    break;
+//                case 5:
+//                    value = _("Configure");
+//                    FcitxDBusMenuAppendProperty(&sub, properties, "label", DBUS_TYPE_STRING, &value);
+//                    /* this icon sucks on KDE, why configure doesn't have "configure" */
+//#if 0
+//                    value = "preferences-system";
+//                    FcitxDBusMenuAppendProperty(&sub, properties, "icon-name", DBUS_TYPE_STRING, &value);
+//#endif
+//                    break;
                 case 6:
                     value = _("Restart");
                     FcitxDBusMenuAppendProperty(&sub, properties, "label", DBUS_TYPE_STRING, &value);
